-- ChatHUD v0.2.2 updated 16Apr2016

-- Project started 27JAN2016; originally called HUChat, never released under that name.
-- I never would have written this if I had found similar functionality elsewhere.  I play under parental controls, though, to make sure I get enough sleep, but I kept missing the disconnect system messages in the chat log, and no one likes to be unexpectedly disconnected...  It doesn't hurt to have other messages more visible as well; I typically ignore the chat box unless I'm trying to figure out what happened.
-- See Curse for change log.

-- Copyright: You may use, copy, modify, publish, distribute, etc. this software (hereafter "it") freely, as long as you:
-- Don't sue or obligate the original author (it is provided "as-is", with no warranty; you assume full responsibility for your use of this open-source code),
-- Don't cheat (don't use it if you are attempting to circumvent a games intended functionality or contrary to any terms or conditions of a game's owner), &
-- Don't plagiarize (you may use it or code from it in derivative works, and don't have to attribute the original creator, but don't say you wrote it all yourself).
-- You do not have to include this notice or use this license in derivative works, but you may not deny anyone else these rights to this release of this code.

-- Customization:  Any customization of this addon, such as changing which channels are displayed and changing display settings, must be done in the LUA file itself.  While this can be done in any text editor, I recommend using one that has features such as line numbers (for debugging) and syntax formatting (for ease of reading); I use Notepad++ myself.

-- Development of this code benefited greatly from these references:
-- * http://www.lua.org/pil/
-- * stolenlegacy (http://wowprogramming.com/forums/development/633)
-- * http://wowwiki.wikia.com/wiki/Handling_events 
-- * various sites found by googling API elements, including wowinterface.com, wowwiki.wikia.com, wowprogramming.com, and probably others I've missed.  These examples were generally limited in scope to how to use a specific API element.
-- * Obviously, this addon would not be possible without the generous API interface offered to addons by Blizzard's World of Warcraft (although it does have its limitations, too).

-- Addon initialization
local addonName = ...  -- The folder name is passed to the addon by WoW when called.
local string = string  -- Creating local links to globabl functions
local strsplit = strsplit  -- may improve call time when using them.
local screen = {}  -- framework for storing environmental variables for internal use.
screen.width, screen.height = strsplit("x",({GetScreenResolutions()})[GetCurrentResolution()],2)  -- For some reason, GetScreenHeight always returns 768, and GetScreenHeight returns a proportionate value that gives the correct aspect ratio at least, but when setting sizes, the actual screen resolution is used; this gets that.
local frame = CreateFrame("Frame",nil,UIParent)  -- Create the frame used for output.
frame:Hide()  -- Hide the frame (until needed).
frame.text = frame:CreateFontString(nil,"OVERLAY","PVPInfoTextFont")  -- Display frame.
frame.temp = frame:CreateFontString(nil,"BACKGROUND",frame.text)  -- Sampling frame.
frame.temp:Hide()  -- This frame is only used to get string sizes, and is never shown.

-- Defaults Section
frame.posn = "BOTTOM"  -- Vertical alignment.
frame.align = "CENTER"  -- Horizontal alignment.
frame.font = "Fonts\\ARIALN.ttf"  -- Display font.
frame.style = "OUTLINE"  -- Font Style.
frame.fade = 1  -- Fade duration in seconds.
frame.fsize = screen.height/20  -- Default (max) font size.
frame.minfsize = screen.height/120 -- If the font size is reduced below this threshold in order to fit messages on the screen, displayed messages are faded immediately instead of keeping it up to be read.  You can always read through all that message traffic in your chat log anyway.  Any time the font size is reduced to fit long messages or high message volume, message duration is also reduced proportionally.  
frame.keep = 1  -- Base seconds to display messages before fading.
frame.wpm = 180  -- Reading speed in words (5-characters) per second; added to default keep time to allow more time to read longer messages.  Decrease to increase additional time for long messages; increase to reduce additional time allowed for long messages.
frame.maxlines = 12  -- Maximum lines to display before deleting oldest; shrink to fit.
frame.maxheight = screen.height/3  -- Max height to display; shrink or delete oldest.
frame.maxwidth = screen.width/2  -- Max width for each line; text reduced to fit.
frame.fromtop = 50  -- Offset from top of screen; default:  1 line of text.
frame.frombottom = 100  -- Offset from bottom; default:  2 lines.
frame.fromleft = 0  -- Offset from left of screen.  Allow maxwidth plus a buffer.
frame.fromright = 0  -- Offset from right.  Leave maxwidth on screen; allow for error.
frame.width = (screen.width - frame.fromleft - frame.fromright)
frame.height = (screen.height - frame.fromtop - frame.frombottom)

-- Reset and update functions (also initialize certain variables)
local function clear(self)  -- Called by functions SetSize and fade.
	self.string = ""  -- Clear display text string.
	self.ttl = self.keep  -- Reset keep duration.
	self.point = self.fsize  -- Reset text size.
	self.text:SetFont(self.font,self.point,self.style)  -- Reset font size.
	self.temp:SetFont(self.font,self.point,self.style)  -- Reset temp size.
	self:Hide()  -- Hide frame (until needed).
end
local function SetSize(self)  -- Sets frame size; used for setting display region offsets from edges of screen; note:  text will not wrap, but will make text display buggy if exceeding this space.  Please allow extra space on the far sides; max text space is set by the maxheight and maxwidth functions.
	self:SetSize(self.width,self.height)  -- Set region size.  
	self:SetPoint("BOTTOMLEFT",self.fromleft,self.frombottom)  -- Set region position; if you scale the UI, you may wish to use a different anchorpoint appropriate to the alignment you are using; I chose this one so the offsets would be positive.  
	self.text:SetJustifyV(self.posn)  -- Set vertical alignment.  
	self.text:SetJustifyH(self.align)  -- Set horizontal alignment.  
	clear(self)  -- Reset and hide display frame.  
	self.text:SetAllPoints()  -- I don't remember why I needed this, but it sure doesn't work without it.  Something about setting points relative to the screen…?
end
local function fade(self,elapsed)  -- Update ttl and fade if negative.
	if (elapsed > .05 or elapsed < 0) then elapsed = .05 end  -- Forces a minimum of 20 calls per fade.  Also avoids immediate fade on initial update calls, which often have an elapsed value of well over 1 second.  
    self.ttl = (self.ttl or self.keep or 4) - elapsed  -- Reduce ttl by elapsed.
    if self.ttl > 0 then return end  -- Return without fading if ttl is positive.
    alpha = 1 + self.ttl / (self.fade or 4)  -- Fade out if ttl has expired.
	if self.ttl <= -(self.fade or 4) then clear(self) else self:SetAlpha(alpha) end  -- Clear and hide frame if fade has completed; otherwise, update alpha value.  
end
frame:SetScript("OnUpdate",fade)

-- Display message function (called to display a new message)
function frame:message(message)  -- Display a new message; called by event handlers.
	if (message == nil) then return end  -- Early exit if message is nil.
	message = string.gsub(message,"]","")  -- Remove opening brackets.
	message = string.gsub(message,"[[]","")  -- Remove closing brackets.
	message = string.gsub(message,"[ ]+"," ")  -- remove any excess space.
	frame.temp:SetText(message)  -- Set temp area to message text for sampling.
	local temp, extra = frame.temp:GetStringWidth(), ""  -- Get width of message.
	if (temp>frame.maxwidth) then -- reduce font size to fit long strings.
		frame.point = min(frame.point, frame.maxwidth/temp * frame.fsize)
	end  -- Now that it fits width-wise, we need to check the length-wise fit:
	if (frame.string ~= "" and frame.string ~= nil) then -- check screen space used.
		frame.string, temp = string.gsub(frame.string .. "\n" .. message,"\n","\n")
		temp = (temp+1)  -- temp = number of lines of text including the new message.
		if (temp*frame.point>frame.maxheight or temp>frame.maxlines) then  -- shrink:
			if (temp<=frame.maxlines or frame.maxlines==0) then  -- fit lines.
				frame.point=min(frame.point,frame.maxheight/temp)  -- shrink to fit.
			else extra, frame.string = strsplit("\n",frame.string,2)  -- delete oldest.
		end	end -- just not in that order…this is vertical shorthand.
	else frame.string = message -- if there wasn't anything in frame.string yet.
	end  -- Now it should be small enough not to clutter the screen too badly. 
	frame.text:SetFont(frame.font,frame.point,frame.style)  -- Set the new font size.
    frame.text:SetText(frame.string)  -- Update text displayed.
	if (frame.ttl <=0) then frame.ttl = 0 end  -- Reset ttl if fading (negative).
	frame:SetAlpha(1)  -- Reset alpha to fully visible, in case it was fading.
    frame:Show()  -- Show the text display frame, if it was previously hidden.
	frame.ttl = frame.ttl + frame.keep  -- Increase ttl by default keep time.
    frame.ttl = frame.ttl + string.len(message)*12/frame.wpm -- Add reading time.
	if (frame.point < frame.minfsize) then frame.ttl = 0 -- fade tiny text immediately.
	elseif (frame.point < frame.fsize) then -- reduce display time by font reduction.
		frame.ttl = frame.point/frame.fsize * frame.ttl
end	end  -- Vertical shorthand again…

local function noRealm(sndr) -- Strip realm from sender name for inclusion in messages; called by event handler functions.
	name, realm = strsplit("-",sndr,2)
	return name
end
local events = {} -- Table of functions defined below to handle specific events.
function events:ADDON_LOADED(msg,...)  -- Completes addon loading, registers events.
	SetSize(frame)  -- Initializes frame size.
	frame:message("|cffaf7fff" .. addonName .. " has loaded!|r")  -- Display loaded msg.
	print("|cffaf7fff" .. addonName .. " has loaded!|r  Settings and customizations can be made directly in the LUA file:  WoW\\Interface\\AddOns\\ChatHUD\\ChatHUD.lua (text file of lua code) is well commented, but contains the actual code of this addon, and you can break it if you try; all changes will be overwritten if you update the addon (so back up your changes).")  -- Display chat log loaded message, with shash command intro.
	for k in pairs(events) do frame:RegisterEvent(k) end -- register all handled events.
	frame:UnregisterEvent("ADDON_LOADED")  -- unregister this one-time event.
end

-- Below are sample event-handler functions for the main chat channels.  There are more listed at various online sources, such as http://wowprogramming.com/docs/events, http://wowwiki.wikia.com/wiki/Events/Communication, http://wow.gamepedia.com/Events/C, and possibly other sites that don't appear in google's top 10 results for multiple chat events.  Adding, removing, and modifying these event handler functions is how you add and remove channels displayed and any additional behavior (such as sounds or filters) for individual chat channels.  /reload in the WoW client will reload the user interface, including recompiling this addon if changes have been saved.  If you get errors for this addon, they should include line numbers and an indication of the nature of the issue, to help with fixing any errors that may have been made in the code.
function events:CHAT_MSG_SYSTEM(msg,...)  -- Handle system messages.
	if string.find(msg,"disconnect") or string.find(msg,"play time expires") then PlaySound("ReadyCheck","SFX") end  -- Play "ReadyCheck" on key terms.
	frame:message("|cffffff00" .. msg .. "|r")  -- Display in yellow text.  Color codes are hexadecimal, ignoring the first two digits (theoretically alpha, but in practice this is ignored) then two digits each for red, green, and blue.  |c escapes the color code, .. is the concatenation operator, and |r returne the color code to what it was before, and is supposed to be used at the end of any custom color range to prevent unexpected bleedover into subsequent text.
end
function events:CHAT_MSG_EMOTE(msg,sndr,...)  -- Handle custom emotes.
	frame:message("|cffff7f00" .. noRealm(sndr) .. " " .. msg .. "|r")
end
function events:CHAT_MSG_TEXT_EMOTE(msg,sndr,...)  -- Handle standard emotes.
	frame:message("|cffff7f00" .. msg .. "|r")
end
function events:CHAT_MSG_GUILD(msg,sndr,...)  -- Handle guild messages.
	frame:message("|cff00ff00" .. noRealm(sndr) .. ": " .. msg .. "|r")
end
function events:CHAT_MSG_OFFICER(msg,sndr,...)  -- Handle guild officer messages.
	frame:message("|cff00ff00" .. noRealm(sndr) .. ": " .. msg .. "|r")
end
function events:CHAT_MSG_PARTY(msg,sndr,...)  -- Handle party messages.
	frame:message("|cffaf7fff" .. noRealm(sndr) .. ": " .. msg .. "|r")
end
function events:CHAT_MSG_PARTY_LEADER(msg,sndr,...)  -- Handle party leader messages.
	frame:message("|cffaf7fff" .. noRealm(sndr) .. ": " .. msg .. "|r")
end
function events:CHAT_MSG_RAID(msg,sndr,...)  -- Handle raid messages.
	frame:message("|cffff7f00" .. noRealm(sndr) .. ": " .. msg .. "|r")
end
function events:CHAT_MSG_RAID_LEADER(msg,sndr,...)  -- Handle raid leader messages.
	frame:message("|cffff7f00" .. noRealm(sndr) .. ": " .. msg .. "|r")
end
function events:CHAT_MSG_WHISPER(msg,sndr,...)  -- Handle whispers.
	frame:message("|cffff7fff" .. noRealm(sndr) .. ": " .. msg .. "|r")
end
function events:CHAT_MSG_BN_WHISPER(msg,sndr,...)  -- Handle battle.net whispers.
	frame:message("|cffff7fff" .. noRealm(sndr) .. ": " .. msg .. "|r")
end
function events:CHAT_MSG_BN_CONVERSATION(msg,sndr,...)  -- Handle battle.net messages.
	frame:message("|cffff7fff" .. noRealm(sndr) .. ": " .. msg .. "|r")
end
function events:CHAT_MSG_BN_INLINE_TOAST_BROADCAST(msg,sndr,...)  -- battle.net misc.
	frame:message("|cffff7fff" .. noRealm(sndr) .. ": " .. msg .. "|r")
end
function events:CHAT_MSG_SAY(msg,sndr,...)  -- Handle local chat.
	frame:message("|cffffffff" .. noRealm(sndr) .. ": " .. msg .. "|r")
end
function events:CHAT_MSG_YELL(msg,sndr,...)  -- Handle Yell messages.
	frame:message("|cffff0000" .. noRealm(sndr) .. ": " .. msg .. "|r")
end
function events:CHAT_MSG_CHANNEL(msg,sndr,lang,chan,tgt,...)  -- Handle channel chat.
	if string.find(msg,UnitName("player")) then -- display the message only if the current player's unit name is included in the message
		frame:message("|cffffffff"..chan..": "..noRealm(sndr)..": "..msg.."|r")
	end
end
frame:SetScript("OnEvent", function(self, event, ...)  -- Calls the appropriate event handler for any registered event received, passing the variable number of arguements that accompany that event.
	events[event](self, ...)
end)
frame:RegisterEvent("ADDON_LOADED")
